/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd;

import java.io.*;
import java.net.*;
import java.util.*;

import com.blommersit.httpd.security.*;

/**
 * This class implements a full webserver, with support for cookies and sessions
 * you can create your own HTTP handlers in Java. Or specify a specific directory
 * that needs to be served. (default = ./wwwpub) <br><br>
 * <b>sample for creating a webserver</b>
 * <pre>
 *    // this method creates a new webserver at port 80
 *    HttpServer s = new HttpServer(80,5,null);
 *    // allow access to a given JAR-resource (this isn't required)
 *    s.getSettings().getAccessFilter().addRule(new AccessFilterRule( AccessFilterRule.RESOURCE_TYPE_JAR_RESOURCE, 0,0, "/com/blommersit/httpd/images",AccessFilterRule.ACTION_ALLOW ));
 *    // set a server variable (this isn't required)
 *    s.setServerVar( "owner", this  );
 *    s.start();
 *    // this is all ;)
 * </pre>
 * <br>
 * <b>Resource Access</b><br/><br/>
 * <p>
 * It's possible to access the resources in the loaded jar files. But be careful
 * enableing this causes some dangerous security hazards: downloading the java
 * .class files. You must set the correct rules (Like in the line above)
 * An example url:  http://localhost/com/blommersit/httpd/public/index.html<br>
 * This is the one available in this library..
 * </p>
 *
 * <b>Creating your own HttpHandler</b><br/><br/>
 * <p>
 * The webserver accepts url's in the following form: http://localhost/com/blommersit/httpd/test/TestHttpHandler
 * when parsing this url it checks if a class of the name com.blommersit.httpd.test.TestHttphandler exists
 * (try it, it exists in this library ;)
 * Here is the source of the TestHttpHandler class:<br>
 * (This sample also demonstrates the use of 'session variables')
 * </p>
 * <pre>
 * import com.blommersit.httpd.*;
 * public class TestHttpHandler implements HttpHandler
 * {
 *    public Object handleHttpRequest(HttpServerThread serverThread, HttpServer server, HttpRequest request, HttpResponse response)
 *    {
 *      Integer count = (Integer)serverThread.getHttpSession().getValue("counter",new Integer(0));
 *      response.println("&lt;html&gt;&lt;head&gt;&lt;title&gt;TEST&lt;/title&gt;&lt;/head&gt;&lt;body bgcolor='#000000' text='#FFFFFF'&gt;");
 *      response.println("&lt;h1&gt;Test&lt;/h1&gt;");
 *      response.println("&lt;p&gt;This is a test page &lt;/p&gt;");
 *      response.println("&lt;b&gt;data send by you:&lt;/b&gt;");
 *      response.println( request.getPost("testdata","") );
 *      response.println("&lt;br&gt;&lt;br&gt;");
 *      response.println("&lt;b&gt;session counter:&lt;/b&gt;");
 *      response.println( count.toString() );
 *      response.println("&lt;/body&gt;&lt;/html&gt;");
 *      serverThread.getHttpSession().setValue("counter",new Integer(count.intValue()+1) );
 *      return null;
 *    }
 * }
 * </pre>
 *
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */

public class HttpServer extends Thread
{
////////////////////////////////////////////////////////////////////////////////
// data members
////////////////////////////////////////////////////////////////////////////////

    /** The tcp/ip port number. The standard value for a webserver is 80 */
    int port;

    /**
     * the maxqueue. This variable defines the number of connections that
     * are maximal queued if they arrive exactly at the same time.
     * This value does NOT represent the maximum number of simultanious
     * connections.
     */
    int maxqueue;

    /**
     * the ip-address that should be used to listen to request
     */
    InetAddress address;

    /**
     * this is the threadgroup name. I don't know why you may want it, but
     * I placed it here
     */
    String threadGroupName;

    /**
     * this is the threadgroup that contains all HttpServerThreads. Every
     * thread request is placed in this threadgroup for easy administration.
     */
    ThreadGroup threadGroup;

    /**
     * The thread counter. This method counts the number of threads that have
     * been started. Technically this number is used to generate an unqiue
     * thread name every time a thread starts.
     */
    int threadCounter=0;

    /**
     * This object contains all HttpServer setttings. This object contains
     * all settings required by the webserver.
     */
    HttpServerSettings serverSettings = new HttpServerSettings();

    /**
     * A session manager is a helper class that is used to maintain a
     * statefull communication  with the clients on the stateless protocol
     */
    HttpSessionManager sessionManager;

    /**
     * This hashmap contains all server variables. The key is the servervariable
     * name. The value is the object stored in the hashmap
     */
    HashMap serverVars = new HashMap();

////////////////////////////////////////////////////////////////////////////////
// contruction / initialisation
////////////////////////////////////////////////////////////////////////////////

    /**
     * the default constructor for the webserver
     * @param port the port number. Port 80 is standard HTTP, but for an application administration
     *          webserver you should use another port. Preferably a port that can be changed
     *          in a settings file.
     * @param maxqueue the queue length. If multiple clients try to connect EXACTLY
     *                          at the same time they get queued. You should set this
     *                          value, because it takes a few ms to create a seperate thread !
     * @param bindaddress the addres to bind to (use null for localhost) Use this for
     *                      pc's with multiple ip's
     */
    public HttpServer( int port, int maxqueue, String bindaddress  )
    {
        super("webserver");
        this.port = port;
        this.maxqueue = maxqueue;
        try
        {
            this.address = InetAddress.getByName(bindaddress);
        }
        catch( Exception e )
        {
            e.printStackTrace();
            try
            {
                this.address = InetAddress.getByName(null);
            }
            catch( Exception e2)
            {
                this.address = null;
                e.printStackTrace();
            }
        }
        threadGroupName = address.toString() + ":" + port;
        threadGroup = new ThreadGroup(threadGroupName);
        sessionManager = new HttpSessionManager(this);
    }

    /**
     * After calling this method, the webserver starts listening to
     * requests.
     */
    public void start()
    {
        super.start();
    }

////////////////////////////////////////////////////////////////////////////////
// server variables
////////////////////////////////////////////////////////////////////////////////

    /**
     * This method sets a server variable. A server variable is a variable
     * that's stored in the webserver. The HttpHandlers can access these
     * variables by a given key. The application creating the webserver
     * should set the server variables before starting the webserver.
     * @param key the name of the server variable to set
     * @param value the value of the server variable
     */
    public void setServerVar( String key, Object value  )
    {
        serverVars.put(key, value);
    }

    /**
     * this method returns a server var
     * @param key the name of the variable name
     * @param def the default value to return if the given server var doesn't
     *              exsist.
     * @return a reference to the given servervar object or the def object
     *          reference if the given variable doesn't exist.
     */
    public Object getServerVar( String key, Object def )
    {
        if( !serverVars.containsKey(key) ) return def;
        return serverVars.get(key);
    }

////////////////////////////////////////////////////////////////////////////////
// getters / setters
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this method returns a reference to the server settings object. This
	 * way it is possible to configure the HttpServer.
	 * @return the server settings object
	 */
	public HttpServerSettings getSettings()
	{
		return serverSettings;
	}

	/**
	 * Sets the current server settings
	 * @param settings the server settings to assign
	 * @since V1.03
	 */
	public void setSettings( HttpServerSettings settings )
	{
		serverSettings = settings;
	}


    /**
     * This method returns a reference the the sessionmanager of the webserver
     * @return the session manager
     */
    public HttpSessionManager getSessionManager()
    {
        return sessionManager;
    }

////////////////////////////////////////////////////////////////////////////////
// client handling
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this method handles a single HTTP request.
	 * @param socket the socket that represents the client.
	 */
	void handleClient(Socket socket)
	{
		HttpServerThread thread = new HttpServerThread(threadGroup, "thread"+(threadCounter++), this,socket);
		thread.start();
	}

    /**
     * This is the main loop of the webserver. NEVER call this method
     * directly, or you will get a blocking webserver
     */
    public void run()
    {
        try
        {
            try
            {
                // create the server socket
                ServerSocket ssocket = new ServerSocket(port, maxqueue, address);
                ssocket.setSoTimeout(5000); // 5 seconds
                try
                {
                    while( !interrupted() )
                    {
                        // read from the socket
                        try
                        {
                            Socket socket = ssocket.accept();
                            handleClient(socket);
                        }
                        catch( java.net.SocketTimeoutException e)
                        {
                            // we need to break out of the accept method once in a while
                            // to check if this thread is interrupted...
                            // no problem ;)
                        }
                    }
                }
                finally
                {
                    ssocket.close();
                    ssocket = null;
                }
            }
            catch( Exception e )
            {
                e.printStackTrace();
            }
        }
        finally
        {
            // interrupt ALL threads
            threadGroup.interrupt();
        }
    }


    /**
     * just for debugging purposes...
     * @param args commandline arguments
     */
    public static void main(String[] args )
    {

System.out.println("Some links for testing:");
System.out.println("httpHandler class: http://localhost/com.blommersit.httpd.test.TestHttpHandler");
System.out.println("gif reosource: http://localhost/com/blommersit/httpd/public/icon48x48.gif");
System.out.println("gif reosource: http://localhost/com/blommersit/httpd/public/index.html");
System.out.println("Great test: http://localhost/com/blommersit/httpd/public/index.html");

        try
        {
            HttpServer s = new HttpServer( 80, 5, null );
            s.getSettings().getAccessFilter().addRule(new AccessFilterRule( AccessFilterRule.RESOURCE_TYPE_JAR_RESOURCE, 0,0, "/com/blommersit/httpd/images",AccessFilterRule.ACTION_ALLOW ));

//s.getSettings().setFileDocumentRoot( "d:/tmp" );
//s.getSettings().getAccessFilter().addRule(new AccessFilterRule( AccessFilterRule.RESOURCE_TYPE_FILESYSTEM, 0,0, "d:/tmp",AccessFilterRule.ACTION_ALLOW ));

            s.start();
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
    }



}
