/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd;

import java.util.*;

import com.blommersit.httpd.log.*;
import com.blommersit.httpd.routes.*;
import com.blommersit.httpd.security.*;
import com.blommersit.httpd.interceptor.HttpInterceptorManager;

/**
 * This class is used to specify the server settings. This class is used
 * by the webserver to know how and what to handle
 * <br>
 * NOTE: There's still a problem with JAR resources: it is possible to
 *       get a listing of the directories... (sorry, need to find a fix for it)
 * <BR>
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */

public class HttpServerSettings
{
////////////////////////////////////////////////////////////////////////////////
// private data members
////////////////////////////////////////////////////////////////////////////////

// SECURITY OPTIONS ////////////////////////////////////////////////////////////

	/**
	 * this is the default access filter. This filter checks the
	 * action to perform <br/>
	 * default: all HttpHandlers allowed the rest isn't !!
	 */
	AccessFilter accessFilter = new AccessFilter();

// ALIASSES ////////////////////////////////////////////////////////////////////

	/**
	 * this is the default alias manager. An alias can be used to
	 * redirect a certain name to another location. This is done by
	 * sending a "301 Moved Permanently". This needs to be done differently.
	 */
	AliasManager aliasManager = new AliasManager();


// FILE-ACCESS-OPTIONS /////////////////////////////////////////////////////////

	/**
	 * This location is used to specify the document root. You can use
	 * '.' to specify the current  directory (application directory)
	 * Filenames are being mapped to this directory, examples:
	 * index.html =&gt;  ./wwwpub/index.html
	 * /test/index.html =&gt; ./wwwpub/test/index.html<br/>
	 * default: ./wwwpub
	 */
	String fileDocumentRoot = "./wwwpub";

	/**
	 * this arraylist contains al index-filenames. The default items
	 * for this array are: index.html and index.htm
	 */
	ArrayList indexFiles = new ArrayList();

// SESSIONS ////////////////////////////////////////////////////////////////////

	/**
	 * Should auto sessionhandling be enabled ? If activated, the
	 * automatic sessionhandling remembers the state between different
	 * HTTP requests. <br/>
	 * default: true
	 */
	boolean autoSessionEnabled=true;

	/**
	 * this is the name of the session key that needs to be used.
	 * This is the name of the cookie send to the browser
	 * default: BIT_SESSION
	 */
	String autoSessionKeyName="BIT_SESSION";

	/**
	 * the session timeout seconds !! If a user doesn't communicate with the
	 * webserver within the given period the session is lost and destroyed.
	 * <br/>
	 * default: 5 minutes
	 */
	int autoSessionTimeout = 5*60;

// FILE-UPLOAD-OPTIONS /////////////////////////////////////////////////////////

	/**
	 * This is the maximum file upload supported by the webserver (2 MB)
	 * @since V1.01
	 */
	long maxFileUpload = 1024*1024*2;

// INTERCEPTORS ////////////////////////////////////////////////////////////////

	/**
	 * The interceptor manager. The place to be to customize headers just before
	 * the content is flushed to the client.
	 * @since V1.03
	 */
	HttpInterceptorManager httpInterceptorManager = new HttpInterceptorManager();


// LOG LISTENER OPTIONS ///////////////////////////////////////////////////////

	/**
	 * This arraylist contains all log listeners. These log listeners will
	 * recieve events from the webserver
	 * @since V1.03
	 */
	LogListenerManager logListenerManager = new LogListenerManager();

// KEEP ALIVE OPTIONS /////////////////////////////////////////////////////////

	/**
	 * How many keep alive requests should be handled per connection
	 * @since V1.03
	 */
	int maxKeepAliveRequestPerConnection =(int)10;

	/**
	 * the keep-alive timeout  in seconds (default 15 seconds)
	 * @since V1.03
	 */
	int keepAliveConnectionTimeout = 15;

// ROUTES OPTIONS //////////////////////////////////////////////////////////////

	/**
	 * the current routes handler
	 * @since V1.03
	 */
	Routes routes = new Routes();

// MISC OPTIONS //////////////////////////////////////////////////////////////

	/**
	 * A different classloader.
	 * When NULL the default class loader is used
	 * @since V1.03
	 */
	ClassLoader classLoader = null;

////////////////////////////////////////////////////////////////////////////////
// construction / initialisation
////////////////////////////////////////////////////////////////////////////////

	/**
	 * The default constructor. It initialises the server settings
	 * object with the default values
	 */
	public HttpServerSettings()
	{
		indexFiles.add("index.html");
		indexFiles.add("index.htm");
	}

////////////////////////////////////////////////////////////////////////////////
// getters and setters
////////////////////////////////////////////////////////////////////////////////

  // Access Filter / security
  ///========================

	/**
	 * this method returns a reference to the accessfilter.
	 * @return the reference to the accessfilter
	 */
	public AccessFilter getAccessFilter()
	{
		return accessFilter;
	}

  // Alias Manager
  //==============

	/**
	 * this method returns the alias manager. An aliasmanager can be
	 * used to make an alias for a given name. NOTE: alias is tested
	 * BEFORE the original resource. Alias 2 alias isn't possible !!
	 * @return a reference to the aliasmanager
	 */
	public AliasManager getAliasManager()
	{
		return this.aliasManager;
	}

  // Document Root
  //==============

	/**
	 * this method returns the file document root.
	 * Filenames are being mapped to this directory, examples:
	 * index.html =&gt;  ./wwwpub/index.html
	 * /test/index.html =&gt; ./wwwpub/test/index.html<br/>
	 * default: ./wwwpub
	 * @return the file document root
	 */
	public String getFileDocumentRoot()
	{
		return fileDocumentRoot;
	}

	/**
	 * this method sets the file document root
	 * @param documentRoot the new document root
	 * @since V1.03
	 */
	public void setFileDocumentRoot( String documentRoot )
	{
		this.fileDocumentRoot = documentRoot;
	}

  // Index Files:
  //============

	/**
	 * this method returns the total number of index files.
	 * @return the number of index files
	 */
	public int getIndexFileCount()
	{
		return indexFiles.size();
	}

	/**
	 * this method returns the given index file a the given location
	 * @param index the number of the index file to retrieve
	 * @return the name of the index file.
	 */
	public String getIndexFile(int index)
	{
		return (String)indexFiles.get(index);
	}

	/**
	 * This method adds an index file. And index file is a filename that
	 * is loaded automaticly if there's only a path supplied and no filename
	 *
	 * The default index files are: index.html and index.htm
	 *
	 * @since V1.03
	 */
	public void addIndexFile( String item )
	{
		indexFiles.add( item );
	}

	/**
	 * inserts an index file at the given position
	 * @since V1.03
	 */
	public void insertIndexFile( int idx, String filename )
	{
		this.indexFiles.add( idx, filename );
	}

	/**
	 * Removes a given index file
	 * @since V1.03
	 */
	public void removeIndexFile( int index )
	{
		indexFiles.remove( index );
	}

	/**
	 * Removes a given index file
	 * @since V1.03
	 */
	public void removeIndexFile( String filename )
	{
		indexFiles.remove( filename );
	}

  // Session Handling:
  //==================

	/**
	 * this method checks if sessionhandling is enabled
	 * @return true if sessionhandling is enabled
	 */
	public boolean getAutoSessionEnabled()
	{
		return autoSessionEnabled;
	}

	/**
	 * enables the auto sessionhandling
	 * @since V1.03
	 */
	public void setAutoSessionEnabled( boolean enabled )
	{
		this.autoSessionEnabled = enabled;
	}



	/**
	 * this method retrieves the default session key
	 * @return the cookie name used by the auto session handler
	 */
	public String getAutoSessionKeyName()
	{
		return autoSessionKeyName;
	}

	/**
	 * Changes the default session name
	 * @since V1.03
	 */
	public void setAutoSessionKeyName( String keyName )
	{
		this.autoSessionKeyName = keyName;
	}

	/**
	 * this method retrieves the session timeout in seconds
	 * @return the session timeout in seconds.
	 */
	public int getAutoSessionTimeout()
	{
		return autoSessionTimeout;
	}

	/**
	 * Change the session timeout
	 * @since V1.03
	 */
	public void setAutoSessionTimeout( int timeout )
	{
		this.autoSessionTimeout = timeout;
	}


  // File uploads
  //=============

	/**
	 * returns the maximum file upload that's supported (in bytes)
	 * @since V1.01
	 */
	public long getMaxFileUpload()
	{
		return maxFileUpload;
	}


	/**
	 * sets the maximum file upload allowed in bytes
	 * @since V1.03
	 */
	public void setMaxFileUpload( long size )
	{
		maxFileUpload = size;
	}

// Interception manager
//=====================

	/**
	 * The interceptor manager. The place to be to customize headers just before
	 * the content is flushed to the client.
	 * @since V1.03
	 */
	public HttpInterceptorManager getHttpInterceptorManager()
	{
		return httpInterceptorManager;
	}


  // Log Listener Manager
  //=====================

	/**
	 * returns the log listener manager. Use this to add your own log listeners
	 *
	 * For example, for adding the a system.out log manager:
	 * <pre>
	 * import com.blommersit.httpd.log.*;
	 * ...
	 * server.getSettings().getLogListenerManager().addListener( new SystemOutLogListener() );
	 * ..
	 *
	 * </pre>
	 * @since V1.03
	 */
	public LogListenerManager getLogListenerManager()
	{
		return logListenerManager;
	}


  // Keep Alive options
  //=====================

	/**
	 * How many keep alive requests should be handled per connection
	 * Default 10
	 * @since V1.03
	 */
	public void setMaxKeepAliveRequestPerConnection( int value )
	{
		maxKeepAliveRequestPerConnection = value;
	}

	/**
	 * How many keep alive requests should be handled per connection
	 * Default 10
	 * @since V1.03
	 */
	public int getMaxKeepAliveRequestPerConnection( )
	{
		return maxKeepAliveRequestPerConnection;
	}

	/**
	 * the keep-alive timeout  in seconds (default 15 seconds)
	 * @since V1.03
	 */
	public void setKeepAliveConnectionTimeout( int value )
	{
		keepAliveConnectionTimeout = value;
	}

	/**
	 * the keep-alive timeout  in seconds (default 15 seconds)
	 * @since V1.03
	 */
	public int getKeepAliveConnectionTimeout(  )
	{
		return keepAliveConnectionTimeout;
	}

  // The Routes
  //===========
	/**
	 * Returns the routes object. For routes handling of http requests
	 * @since V1.03
	 */
	public Routes getRoutes()
	{
		return routes;
	}

  // Misc
  //===========
	/**
	 * Sets the class loader
	 * @since V1.03
	 */
	public void setClassLoader ( ClassLoader classLoader )
	{
		this.classLoader = classLoader;
	}

	/**
	 * Returns the different classloader
	 * @since V1.03
	 */
	public ClassLoader getClassLoader()
	{
		return classLoader;
	}


}
