/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd;

import java.io.*;
import java.net.*;

import com.blommersit.httpd.log.*;
import com.blommersit.httpd.security.*;

/**
 * This is the thread that handles a single client request.
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */

public class HttpServerThread extends Thread
{
////////////////////////////////////////////////////////////////////////////////
// data members
////////////////////////////////////////////////////////////////////////////////

    /**
     * the socket to use with communications with the client
     */
    Socket socket;

    /**
     * a reference to the server object
     */
    HttpServer server;

    /**
     * the session variable associated with this request
     */
    HttpSession session;


	/**
	 * The current httprequest en response
	 * @since V1.03
	 */
	HttpRequest     request = null;


	/**
	 * The current httprequest en response
	 * @since V1.03
	 */
	HttpResponse    response = null;


////////////////////////////////////////////////////////////////////////////////
// construction / initialisation
////////////////////////////////////////////////////////////////////////////////

    /**
     * the default constructor. Only used by the webserver
     * @param group the thread group this thread belongs to
     * @param name the name of the thread
     * @param server a reference to the webserver
     * @param socket the socket of the client
     */
    public HttpServerThread( ThreadGroup group, String name, HttpServer server, Socket socket )
    {
        super( group, name );
        this.server = server;
        this.socket = socket;
    }

////////////////////////////////////////////////////////////////////////////////
// public functions
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this method returns the HttpSession object
	 * @return the httpsesion object. NOTE if autosessions are disabled null
	 *   is returned.. so take care !!
	 */
	public HttpSession getHttpSession()
	{
		return session;
	}

	/**
	 * this method returns the http server object. Required for accessing
	 * settings and other options
	 * @since V1.03
	 */
	public HttpServer getServer()
	{
		return this.server;
	}

	/**
	 * Returns the current httprequest. Warning: you can only use this after
	 * the request has been succesfully parsed
	 * @since V1.03
	 */

	public HttpRequest getHttpRequest()
	{
		return this.request;
	}

	/**
	 * Returns the current response. Warning: you can only use this after
	 * the request has been succesfully parsed
	 * @since V1.03
	 */

	public HttpResponse getHttpResponse()
	{
		return this.response;
	}


////////////////////////////////////////////////////////////////////////////////
// Specific HTTP request handlers
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this method searches for a HttpHandler with the given packagename
	 * http://localhost/com.blommersit.packagename.etc.HttpHandler
	 * <p>
	 * Since V1.03 this method first tries to match the routed requests
	 * </p>
	 *
	 * @param request the request object
	 * @param response the response object
	 * @return true if the handler is found and called else false
	 */
	boolean handlePackageNameRequest( HttpRequest request, HttpResponse response )
	{
		String routedPackage = server.getSettings().getRoutes().findRoute( request );
		String location = null;
		if( routedPackage != null )
		{
			location = routedPackage;
		}
		else
		{
			// Everything ok??  Search the correct package
			// 1. packagename of the correct HttpHandler class
			//    package seperator can be '/' and '.' zijn
			location = request.getLocation();
			// remove the first '/'
			if (location.length() > 1 && location.charAt(0) == '/') location = location.substring(1);
			location = location.replaceAll("\\/", ".");
		}
		boolean found=false;
		try
		{
			// try to find the HttpHandler interface for the given class
			Class cls = null;

			if( server.getSettings().getClassLoader() != null )
			{
				cls = Class.forName( location, true, server.getSettings().getClassLoader() );
			}
			else
			{
				cls = Class.forName( location );
			}

			if (isInterface(cls,HttpHandler.class))
			{
				Object o = cls.newInstance();
				((HttpHandler)o).handleHttpRequest(this,server,request,response);
				server.getSettings().getLogListenerManager().notify_logRequest( this, request, response, "class", location );
				return true; // we're done...
			}
		}
		catch( ClassNotFoundException x )
		{
			// class not found, no problem!
		}
		catch( Exception e)
		{
			server.getSettings().getLogListenerManager().notify_logError( this, request, response, "Exception handling (java) request: " + e.getMessage() );
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * Better interface detection, now all parent interfaces are checked  for
	 * the given interface!
	 * This piece of code is supplied by Gennadiy from the Ukraine.
	 * @param cls the class
	 * @return true if the class implements the given interface
	 * @since V1.02
	 */

	boolean isInterface(Class cls, Class interfaceToFind)
	{
		Class[] interfaces = cls.getInterfaces();
		for( int i=0; i < interfaces.length; i++ )
		{
			if( interfaces[i] == interfaceToFind )
			{
				return true;
			}
		}
		if (cls.getSuperclass()==null)
		{
			return false;
		}
		return isInterface(cls.getSuperclass(), interfaceToFind);
	}


	/**
	 *  this method tries to handle a java 'resource' request
	 *  A resource request is a filename in the existing jarfiles...
	 *  For example: http://localhost/images/logo.gif
	 *  Pretty cool, huh !!<br/>
	 *  To secure it you can restrict access by specifying several items<br/>
	 *  <b>Warning:</b> At the moment it's possible to retrieve the .class files
	 *  and package contents. This needs to be fixed in the future
	 * @param request the request object
	 * @param response the response object
	 * @return true if the resource is found and returned else false
	 * @throws IOException it could happen when reading the resource
	 */
	boolean handleResourceRequest( HttpRequest request, HttpResponse response ) throws IOException
	{
		// get the class loader
		ClassLoader loader = ClassLoader.getSystemClassLoader();

		// find the resource
		String location = request.getLocation();
		if( location.length() > 1 && location.charAt(0)=='/' ) location = location.substring(1);

		// open the resource stream
		InputStream stream = loader.getResourceAsStream(location);
		if( stream == null ) return false;

		// set the corrent header
		response.setHeader("content-type:",MimeTypeDetector.getMimeType( location, response, stream ));

/** TODO: Content length for JAR resources is HARD !!
		response.setHeader("content-length:", location);

TODO remove "keepAlive = false" when resource request sent file sizes
in the call of this handler!!

*/
		// retrieve the bytes and send them
		try
		{
			// write the stream
			//response.writeStream( stream );


			// Not very nice, but I don't know of another method to get the
			// size of this resource. This is required to allow keep-alive
			byte[] resource = Utils.getStreamAsBytes( stream );

			// close the file
			stream.close();

			// write the resource
			response.setHeader("content-length:",Integer.toString( resource.length ) );
			response.flush();	// Is required before direct writing !!!!

			response.writeDirect( resource, 0, resource.length );

//            // set the length
//            response.setHeader("accept-ranges:","bytes");
//            response.setHeader("content-length:",Integer.toString(count) );

			server.getSettings().getLogListenerManager().notify_logRequest( this, request, response, "jar", location);
		}
		catch( Exception e)
		{
			server.getSettings().getLogListenerManager().notify_logError( this, request, response, "Exception handling (jar) request: " + e.getMessage() );
			e.printStackTrace();
		}
		return true;
	}

	/**
	 * this method tries to handle a file request
	 * A resource request is a filename in the document root directory
	 * For example: http://localhost/images/logo.gif
	 * @param request the request object
	 * @param response the response object
	 * @return true if the file is found and returned else false
	 * @throws IOException it could happen when reading the resource
	 */
	boolean handleFileRequest( HttpRequest request, HttpResponse response ) throws IOException
	{
		// make a filename from the request
		String location = request.getLocation();
		if( !location.startsWith("/") ) location = "/" + location;

		// build the complete filename
		String filename = server.getSettings().getFileDocumentRoot() + location;
		File file = new File(filename);
		if( !file.exists() || file.isDirectory() )
		{
			if( !filename.endsWith("/") ) filename += "/";
			String orgfilename=filename;
			for( int i=0; i < server.getSettings().getIndexFileCount() && (!file.exists()||file.isDirectory()) ; i++ )
			{
				filename = orgfilename + server.getSettings().getIndexFile(i);
				file = new File( filename );
			}
			if( !file.exists() || file.isDirectory() )
			{
				//server.getSettings().getLogListenerManager().notify_logError( this, request, response, "File not found: " + file.getAbsolutePath() );
				server.getSettings().getLogListenerManager().notify_logDebugMessage( this, request, response, HttpServerLogListener.DEBUG_LEVEL_MIN, "File not found: " + file.getAbsolutePath() );
				return false;
			}
		}

		// build absolute path (without) '. and '..'  and check if the user can access the directory
		// Prevent '..' and unicode tricks
		// SINCE v1.01
		file = file.getCanonicalFile();
		File root = (new File( this.server.getSettings().getFileDocumentRoot() )).getCanonicalFile();
		String sFile = file.getPath();
		String sRoot = root.getPath();
		// NOT SAFE??
		if(!sFile.startsWith(sRoot) )
		{
			server.getSettings().getLogListenerManager().notify_logError( this, request, response, "Security error, path doesnt start with root: " + root.getAbsolutePath());
			//System.out.println("Denied Request: " + sFile );
			return false;
		}

		try
		{
			response.writeFile( file );
			server.getSettings().getLogListenerManager().notify_logRequest( this, request, response, "file", file.getAbsolutePath() );
		}
		catch( Exception e)
		{
			server.getSettings().getLogListenerManager().notify_logError( this, request, response, "Exception handling (file) request: " + e.getMessage() );
			e.printStackTrace();
		}
		return true;
	}

////////////////////////////////////////////////////////////////////////////////
// here we handle the HTTP request
////////////////////////////////////////////////////////////////////////////////

	/**
	 * the main run method, that handles the client request
	 * This method is called by the webserver. You
	 * never should call it manually.
	 */
	public void run()
	{
		try
		{
			try
			{
//System.out.println("Handling Client: " + socket.getInetAddress().toString() );
				LogListenerManager log = server.getSettings().getLogListenerManager();

				// set the timeout settings
				if( server.getSettings().getMaxKeepAliveRequestPerConnection() > 0 )
					socket.setSoTimeout( server.getSettings().getKeepAliveConnectionTimeout() * 1000 );
				else
					socket.setSoTimeout( 30000 ); // 30 sec

				// retrieve the data input stream
				DataInputStream input = new DataInputStream(socket.getInputStream());
				DataOutputStream output = new java.io.DataOutputStream( socket.getOutputStream() );
				log.notify_logDebugMessage(this,null,null, HttpServerLogListener.DEBUG_LEVEL_MIN,"=================================================");
				log.notify_logDebugMessage(this,null,null, HttpServerLogListener.DEBUG_LEVEL_MIN,"Starting request handling for " + socket.getInetAddress().toString());
				log.notify_logDebugMessage(this,null,null, HttpServerLogListener.DEBUG_LEVEL_MIN,"=================================================");

				int requestCounter = 0;
				boolean keepAlive = false;
				do
				{
					requestCounter++;

					// create the request and response
					request = new HttpRequest(this);
					response= new HttpResponse(this,output);

					log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Handle Request " + (requestCounter+1) );
					log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"-------------------------------" );

					// init the request object
					int result = request.init( input, socket, requestCounter > 0 );

					if( result != HttpUtils.HTTP_OK )
					{
						if( result != 0 )
						{
							HttpUtils.sendErrorMessage( response , result );
							server.getSettings().getLogListenerManager().notify_logError( this, request, response, "Error initializing request!"  );
						}
						return;
					}

					// load the session vars
					log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Loading session variables.");
					session = server.getSessionManager().loadSessionVars(request,response);

					// keepalive ?
					//keepAlive	= !request.getHeader("keep-alive:","").equals("");	// IE Doesn't sent a keep-alive header !!!!
					keepAlive = ( request.getHeader("connection:","").toLowerCase().startsWith("keep-alive") );
					if( keepAlive )
					{
						log.notify_logDebugMessage(this, request, response,HttpServerLogListener.DEBUG_LEVEL_MIN,"Using persistent connections(request nr=" +(requestCounter + 1) + ") .");
					}

					// perform the alias filter
					do // required for performing a break
					{
						log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Checking Aliasses.");

						boolean bResult = server.getSettings().getAliasManager().redirectLocationIfRequired( request, response );
						if( bResult )
						{
							log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Alias found, redirect to: " + response.getHeader("location:",""));
							server.getSettings().getLogListenerManager().notify_logRedirect( this, request, response, response.getHeader("location:","") );
							break;
						}

						try
						{
							// try the HTTP handler 'handler'
							if( server.getSettings().getAccessFilter().getAction(AccessFilterRule.RESOURCE_TYPE_JAVA_HANDLER, request)==AccessFilterRule.ACTION_ALLOW )
							{
								// handle the routes
								log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Activating Java handler.");
								bResult = handlePackageNameRequest( request, response );
								if( bResult )
								{
									keepAlive = false;	// keepAlive is only possible when buffered!
									break; // we're done
								}
							}
							else
							{
								log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Java handler disabled: activate with AccessFilter!");
							}

							// try the resource handler
							if( server.getSettings().getAccessFilter().getAction(AccessFilterRule.RESOURCE_TYPE_JAR_RESOURCE, request)==AccessFilterRule.ACTION_ALLOW)
							{
								log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Activating JAR Resource handler.");

								bResult = handleResourceRequest( request, response );
								if( bResult ) break;
								//{
								//	keepAlive = false; // keepAlive is only possible when buffered!
								//	break;
								//}
							}
							else
							{
								log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"JAR Resource handler disabled: activate with AccessFilter!");
							}

							// try the file handler (corrected it Thanks Gennadiy)
							if( server.getSettings().getAccessFilter().getAction(AccessFilterRule.RESOURCE_TYPE_FILESYSTEM, request)==AccessFilterRule.ACTION_ALLOW)
							{
								log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Activating File handler.");
								bResult = handleFileRequest( request, response );
								if( bResult ) break; // we're done
							}
							// NOTE: This log is only required if file access is disabled. File access already logs a file not found exception!
							else
							{
								log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"File handler disabled: activate with AccessFilter!");
							}

							// tell the user we didn't find the page
		/**@todo this needs to be a 404 page, that's given in the ServerSettings, at the moment you should do it with this ;) */
							log.notify_logDebugMessage(this,request,response, HttpServerLogListener.DEBUG_LEVEL_MIN,"Executing 404 Code.");

							HttpUtils.sendErrorMessage( response, HttpUtils.HTTP_NOT_FOUND );
							server.getSettings().getLogListenerManager().notify_logError( this, request, response, "Requested url not found!" );
							keepAlive = false;	// is required on a 404 becease Mozilla keeps waiting for more response...(?)
						}
						finally
						{
							// Ready ? Always flush de data
							response.flush();
						}
					}
					while( false );	// Required to easily break to the correct location
				}
				while( keepAlive && requestCounter < server.getSettings().getMaxKeepAliveRequestPerConnection() );

				// done ?
			}
			catch( Exception e)
			{
				e.printStackTrace();
			}
		}
		finally
		{
			// always close the socket
			try
			{
				socket.close();
			}
			catch( Exception e)
			{
				e.printStackTrace();
			}
		}
	}
}
