/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd;

import java.util.*;

import com.blommersit.httpd.log.*;
import com.blommersit.httpd.util.*;

/**
 * This class manages sessions. The class automaticly cleans up the sessions
 * if expired and makes it possible to retrieve the sessions for a given user.
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */
public class HttpSessionManager
{
////////////////////////////////////////////////////////////////////////////////
// data members
////////////////////////////////////////////////////////////////////////////////

    /**
     * this hashmap contains all 'HttpSession' objects
     */
    HashMap sessions = new HashMap();

    /**
     * the last given session id
     */
    long sessionCounter=0;

    /**
     * the server var. Just for administration purposes
     */
    HttpServer server;

    /**
     * the sesion garbage collector. Yep I had to create my own one.
     */
    SessionGarbageCollector gc;

////////////////////////////////////////////////////////////////////////////////
// helping functions
////////////////////////////////////////////////////////////////////////////////

    /**
     * the default constructor. Only used internally by the webserer
     * @param server the server object this session manager belong to
     */
    public HttpSessionManager(HttpServer server)
    {
        this.server = server;
        gc = new SessionGarbageCollector();
        gc.start();
    }


    /**
     * this method generations unique session id's
     * @todo not yet very secure, we need to use MD5 and a more randomness, to make it harder
     *      to guess the sessionid
     * @return the unique session id
     * @since V1.01 added better security with the help of MD5
     */
    public String generateNewSessionId()
    {
        // the result
        StringBuffer result = new StringBuffer(32);

        // increase the session counter
        sessionCounter++;


        // append the date/time
        //result.append(System.currentTimeMillis());

        // generate random part
        result.append( CryptUtils.generateRandom32() ) ;

        // append the unique id (this is required, only using MD5 isn't good
        // enought, we don't have the garantee every MD5 is unique... And we
        // don't want to mix up sessions!
        result.append(Long.toHexString(sessionCounter));

        return result.toString();
    }

////////////////////////////////////////////////////////////////////////////////
// public functions
////////////////////////////////////////////////////////////////////////////////

    /**
     * this method remove a given session.
     * @param session the session to destroy
     */
    public void destroySession( HttpSession session )
    {
        sessions.remove(session);
    }


	/**
	 * this method is called to load the sessions for the given request.
	 * If the client does't have a session a new one is created.
	 * <br/:>
	 * This method is used internally by the webserver.
	 * @param request the request object
	 * @param response the reponse object
	 * @return null if autosessions are disabled, else the HttpSession object
	 *      is returned.
	 */
	public HttpSession loadSessionVars( HttpRequest request, HttpResponse response )
	{
		// auto session enabled ??
		if( server.getSettings().getAutoSessionEnabled() )
		{
			// this is the session we need to return
			HttpSession session=null;

			// first try to find the given cookie
			boolean found=false;
			String id = request.getCookie(server.getSettings().getAutoSessionKeyName(),"");
			if( !id.equals("") )
			{
				// check if the cookie still exists
				synchronized( sessions )
				{
					// check if the session contains the key
					if( sessions.containsKey(id) )
					{
						// check for timeout
						session = (HttpSession)sessions.get(id);
						if( !session.isExpired(server) )
						{
							session.access();
							found = true;
						}
						else sessions.remove(session);
					}
				}
			}
			// not found ? Create the session
			if( !found )
			{
				synchronized( sessions )
				{
					// create the session object
					session = new HttpSession();
					id = generateNewSessionId();
					sessions.put(id,session);
					response.setCookie(server.getSettings().getAutoSessionKeyName(),id);
				}
			}
			return session;
		}
		return null;
	}

	/**
	 * This method returns the session
	 * @since V1.03
	 */
	public HashMap getSessions()
	{
		return sessions;
	}

////////////////////////////////////////////////////////////////////////////////
// Session garbage collector
////////////////////////////////////////////////////////////////////////////////


    /**
     * this thread runs every x-seconds to check if sessions have expired, if
     * they have they are removed. A kind of garbage collector ;)
     */
    class SessionGarbageCollector extends Thread
    {
        /** constuction */
        public SessionGarbageCollector()
        {
            super();
            try
            {
                this.setDaemon(true);
            }
            catch( Exception e )
            {
                e.printStackTrace();
            }
        }
        /** the main 'check' loop. Garbage collects every 30 seconds */
        public void run()
        {
            try
            {
                while( !interrupted() )
                {
                    sleep(30000);
                    synchronized( sessions )
                    {
                        // check ALL sessions
                        Iterator i = sessions.keySet().iterator();
                        while( i.hasNext() )
                        {
                            // get the session
                            String key = (String)i.next();
                            HttpSession session = (HttpSession)sessions.get(key);

                            // timeout happend
                            if( session.isExpired(server) )
                            {
                                i.remove(); // Fixed memory leak, thanks Gabriel Klein!
                            }
                        }
                    }
                }
            }
            catch( InterruptedException e )
            {
                interrupt();
            }
        }
    }

}
