/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/
package com.blommersit.httpd;

import java.io.*;
import java.util.*;

/**
 * This helping class is used to retrieve the auto generated version info
 * We use a version format of the following form:<BR/>
 * MM.mm.rrr.bbbb<BR/>
 * MM  = major version<BR/>
 * mm  = minor version<BR/>
 * rrr = revision version<BR/>
 * bbbb= revision build number<BR/>
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.01
 */
public class VersionInfo
{
    /** this method returns the name of the application */
    public String getName()
    {
        return "Blommers IT - Java HTTP Server Library";
    }


////////////////////////////////////////////////////////////////////////////////
// build information
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this internal properties object is used to 'remember' the version
	 */
	 Properties props = null;

	/**
	 * this method returns the specified item from the build info. This method
	 * opens the file build.num, this file can be automaticly increment with
	 * the correct JBuilder plugin
	 * @param item the item to retrieve, can be:<BR>
	 *        'version' = current version mm.mm.rrr.bbbbbbbbbbb<BR>
	 *        'major' = major version<BR>
	 *        'minor' = minor version<BR>
	 *        'revision' = revision number<BR>
	 *        'build' = total build number<BR>
	 *        'vrbuild' = buildnumber of this Version/Revision number<BR>
	 *        'date' = full GMT build date
	 *        'epoch' = UNIX epoch date (can be passed as integer to the date constructor)
	 * @param def the default item to return if none found
	 */
	public String getBuildInfo( String item, String def )
	{
		// load the info once
		if( props == null )
		{
			props = new Properties();
			InputStream in = null;
			try
			{
				in = this.getClass().getResourceAsStream("build.num");
				props.load( in );
				in.close();
			}
			catch ( Exception ex )
			{
			}

			// parse the version
			String version = props.getProperty("version","");
			String[] items = Utils.explode(".", version );
			props.setProperty("major","?");
			props.setProperty("minor","??");
			props.setProperty("revision","???");
			props.setProperty("vrbuild","????");
			if( items.length > 0 ) props.setProperty("major",items[0]);
			if( items.length > 1 ) props.setProperty("minor",Utils.makeString(items[1],2,'0',false) );
			if( items.length > 2 ) props.setProperty("revision",Utils.makeString(items[2],2,'0',false));
		}

		// return the propertie
		return props.getProperty(item,def);
	}

	/** this method returns the total build number */
	public String getTotalBuildNumber()
	{
		return getBuildInfo( "build", "0" );
	}

	/** this method returns the version string */
	public String getBuildVersion()
	{
		return getBuildInfo( "major", "?" ) + "." + getBuildInfo( "minor", "??" ) + "." + getBuildInfo("revision","???");
	}

	/**
	 * a static method to retrieve the build time as an integer. This is in
	 * the format of HHMMSS
	 */
	public int getBuildTime()
	{
		long value = Long.parseLong(getBuildInfo("epoch","0"));
		//Date date = new Date( value );
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis( value );
		return c.get(Calendar.HOUR)*10000+c.get(Calendar.MINUTE)*100+c.get(Calendar.SECOND);
	}

	/**
	 * a static method to retrieve the data as an integer. This is an ansi date
	 * YYYYMMDD
	 */
	public int getBuildDate()
	{
		long value = Long.parseLong(getBuildInfo("epoch","0"));
        Calendar c = Calendar.getInstance();
        c.setTimeInMillis( value );
        return c.get(Calendar.YEAR)*10000 + c.get(Calendar.MONTH)*100 + c.get(Calendar.DAY_OF_MONTH);
	}

	/**
	 * this method returns a full version number. In the form
	 * M.m.R.B (where M=major, mm in minor, RRR=revision, B is build number
	 */
	public String getCompleteVersion()
	{
		return getBuildInfo("version","??");
	}

	/**
	 * this method returns the full date
	 */
	public String getCompleteDate()
	{
		return getBuildInfo("date","??");
	}

	/** this method returns the major version */
	public int getMajor()
	{
		return Utils.parseInt(props.getProperty("major","0"),10,0);
	}

	/** this method returns the minor version */
	public int getMinor()
	{
		return Utils.parseInt(props.getProperty("minor","0"),10,0);
	}

	/** this method returns the minor version */
	public int getRevision()
	{
		return Utils.parseInt(props.getProperty("revision","0"),10,0);
	}

	/** this method returns the vrbuild version */
    public int getVersionBuild()
    {
		return Utils.parseInt(props.getProperty("vrbuild","0"),10,0);
	}

}
