/*
					  Blommers IT - Webserver Library
	Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
	-----------------------------------------------------------------------

	This program is free software; you can redistribute it and/or modify it
	under the terms of the  Blommers IT - Free Software License Version 1.0
	as published by Blommers IT;  either version 1 of the License,   or (at
	your option) any later version.
	This program is  distributed in the  hope that  it will be useful,  but
	WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
	for more details.

	You should have  received  a copy of the   Blommers IT -  Free Software
	License along with this program; if not, write to
			Blommers IT, de Amert 124,
			5462 GH, Veghel, The Netherlands
	or request for the license by sending an e-mail to info@blommers-it.nl
	or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd.routes;

import java.util.*;
import java.util.regex.*;
import com.blommersit.httpd.*;

/**
 * This class represents a single Route. See the routes class for more info.
 * <p>
 * Every variable name starts with a "$"
 * Every catch-all starts with a "*". A catch all can only be used at the end
 * of the line!
 * </p>
 *
 * <b>Route Samples:</b>
 * <pre>
 * '' : 'http:/localhost/' Root Domainname
 *
 * </pre>
 * @todo Test the route code, it isn't tested very good. Write some unit test or something
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.03
 */
public class Route
{
////////////////////////////////////////////////////////////////////////////////
// Data Members
////////////////////////////////////////////////////////////////////////////////

	/** the route description */
	String routeSource;

	/** the parsed route description (speed improvement)*/
	transient String[] sourceItems;

	/** the requirements */
	HashMap requirements = new HashMap();

	/** the default source values */
	HashMap defaults = new HashMap();

	/** the target class */
	String targetHandler;

	/** the extra target get fields */
	HashMap targetGetFields = new HashMap();

////////////////////////////////////////////////////////////////////////////////
// Public Interface / Application Interface
////////////////////////////////////////////////////////////////////////////////

	/**
	 * the default constructor
	 */
	public Route( String routeSource, String targetHandler )
	{
		// if the route ends with a '/' we need to remove it
		if( routeSource.length() > 1 && routeSource.charAt( routeSource.length()-1 ) == '/' ) routeSource = routeSource.substring(0,routeSource.length()-1);

		// if the route starts with a '/' we need to remove it
		if( routeSource.length() > 0 && routeSource.charAt(0) == '/' ) routeSource = routeSource.substring(1);

		this.routeSource = routeSource;
		this.targetHandler = targetHandler;
		sourceItems = Utils.explode( "/", this.routeSource );
	}

	/**
	 * this method adds a requirement for a given variable
	 */
	public void setRequirement( String varName, String requirement )
	{
		requirements.put( varName, requirement );
	}

	/**
	 * this method sets the default value for a given variable
	 * @param varName The variable name to set
	 * @param value The Default value to use.
	 */
	public void setDefault( String varName, String value )
	{
		defaults.put( varName, value );
	}


	/**
	 * Call this method to add an extra request parameter
	 * @return String
	 */
	public void addGetValue( String varName, String value )
	{
		this.targetGetFields.put( varName, value );
	}


////////////////////////////////////////////////////////////////////////////////
// Internal Interface
////////////////////////////////////////////////////////////////////////////////

	/** returns the target url */
	public String getTargetHandler()
	{
		return targetHandler;
	}

	/**
	 * this method tries to evaluate the given sourcePath (split in components)
	 * with the described route
	 */
	public boolean matchesRoute( String[] pathParts, HttpRequest request, HashMap varValues )
	{
		int maxCount = Math.max( sourceItems.length, pathParts.length );
		for( int i = 0; i < maxCount; i++ )
		{
			// haal de waardes op
			String sourceItem = (i < sourceItems.length)?sourceItems[ i ]:null;
			String pathItem = (i < pathParts.length)?pathParts[ i ]:null;

			// No source item? Then we didn't find a match
			if( sourceItem == null ) return false;

			// parse the source item
			int firstChar = sourceItem.length() > 0 ? sourceItem.charAt( 0 ) : (int) ' ';
			// if( value.equals("") ) continue;	// skip empty items

			// variable name
			if( firstChar == '$' )
			{
				// if the pathitem is null and a default value is
				boolean hasDefaultValue	= this.defaults.containsKey( sourceItem );
				if( hasDefaultValue && pathItem == null )
				{
					pathItem = (String)this.defaults.get( sourceItem );
				}

				// no default and no target item, then we have to go
				if( pathItem == null && !hasDefaultValue ) return false;

				String requirement = (String)requirements.get( sourceItem );
				if( requirement != null )
				{
					// null path can never match a requirement
					if( pathItem == null ) return false;
					if( !pathItem.matches( requirement ) ) return false;
				}

				// yes we found a variable
				if( pathItem != null )
				{
					varValues.put(sourceItem, pathItem);
				}
			}
			// catch all?
			else if( firstChar == '*' )
			{
				break;	// we're done!
			}
			// a normal path part
			else if( !sourceItem.equals( pathItem ) )
			{
				return false;
			}
		}
		// add the extra get fields
		Iterator iter = targetGetFields.keySet().iterator();
		while( iter.hasNext() )
		{
			String key = (String)iter.next();
			if( !varValues.containsKey(key) )
			{
				varValues.put( key, targetGetFields.get( key ) );
			}
		}
		return true;
	}


}
