/*
					  Blommers IT - Webserver Library
	Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
	-----------------------------------------------------------------------

	This program is free software; you can redistribute it and/or modify it
	under the terms of the  Blommers IT - Free Software License Version 1.0
	as published by Blommers IT;  either version 1 of the License,   or (at
	your option) any later version.
	This program is  distributed in the  hope that  it will be useful,  but
	WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
	for more details.

	You should have  received  a copy of the   Blommers IT -  Free Software
	License along with this program; if not, write to
			Blommers IT, de Amert 124,
			5462 GH, Veghel, The Netherlands
	or request for the license by sending an e-mail to info@blommers-it.nl
	or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd.routes;

import java.util.*;

import com.blommersit.httpd.*;
import com.blommersit.httpd.log.*;

/**
 * This class is used to manages routes. A route is a kind of alias for accessing
 * another HttpHandler. For this design I've been inspired by Ruby on Rails ;-)
 * Only the PATH part of the url is handled!
 * <p>
 * For example:
 * </p>
 *
 * The route belows maps all requests to "images/filename" to a httphandler
 * which retrieves the requested image via a database.
 *
 * <pre>
 * Route route = new Route( "images/$filename", "com.blommersit.ImageHandler" );
 * route.setRequirement( "$filename", "[A-Za-z_][A-Za-z0-9_]* );	// optional a regular expression
 * routes.add( route );
 * </pre>
 *
 * Some request samples for the code above:
 * <pre>
 *  "images/test.gif" :  "com.blommersit.ImageHandler?filename=test.gif"
 *  "images/test.gif" :  "com.blommersit.ImageHandler?filename=test.gif"
 * </pre>
 *
 * There's also a special route part *name catches all following parameters.
 * The catch all can only be used at the end of the line!
 * For example:
 * <pre>
 *  new Route( "blogs/*rest", "com.blommersit.BlogHandler" )
 * </pre>
 *
 * The request above handlers:
 * <pre>
 *  "blogs/2006/12/14" : "com.blommersit.BlogHandler"
 *  "blogs" : "com.blommersit.BlogHandler"
 * </pre>
 *
 * <p>
 * At the moment the routes are handled before all other handlers. Also the
 * routes currently are only tested for HttpHandlers. You cannot use them for
 * resources or files.
 * </p>
 *
 * <p>
 * The route variables ALWAYS overrule the old GET parameters supplied to
 * the query! Even the default values.
 * </p>
 *
 * <p>
 * Aliasses are handled before the route requests!
 * </p>
 *
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.03
 */
public class Routes
{
	/** The current routes */
	ArrayList routes = new ArrayList();

	/** adds a route */
	public void add( Route route )
	{
		this.routes.add( route );
	}

	/**
	 * this method validates a route and returns the Classname if the
	 * route is found. It also modifies the request query string with the
	 * parameters of the given route
	 * @return the classname of the matched route or null if no route was found
	 */
	public String findRoute( HttpRequest request )
	{
		// retrieve the location and remove the starting slash
		String location = request.getLocation();
		if( location.length() > 0 && location.charAt(0) == '/' ) location = location.substring(1);
		if( location.length() > 0 && location.charAt( location.length() -1 ) == '/' ) location = location.substring(0,location.length()-1);

		// explode the URL path in the given components
		String[] requestPath = Utils.explode( "/", location );

		HashMap varValues = new HashMap();
		for( int i=0; i < routes.size(); i++ )
		{
			varValues.clear();	// clear the varvalues
			Route route = (Route)routes.get( i );
			if( route.matchesRoute( requestPath, request, varValues ) )
			{
				// copy the var variables to the request object
				Iterator iterator = varValues.keySet().iterator();
				while( iterator.hasNext() )
				{
					String varName = (String)iterator.next();
					String value = (String)varValues.get( varName );
					varName = varName.substring(1);
					request.setGet( varName, value );
				}
				request.setRoute( route );
				return route.getTargetHandler();
			}
		}

		return null;
	}

}
