/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd.security;

import com.blommersit.httpd.*;

/**
 * This class represents a single access rule. An accessrule describes
 * what to do on given request location. The rule describes a PREFIX
 * of the location.
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */
public class AccessFilterRule
{
////////////////////////////////////////////////////////////////////////////////
// constants
////////////////////////////////////////////////////////////////////////////////

    /** the different resource types, this rules applies for */
    public final static int RESOURCE_TYPE_JAVA_HANDLER = 1,
                            RESOURCE_TYPE_JAR_RESOURCE = 2,
                            RESOURCE_TYPE_FILESYSTEM = 4,
                            RESOURCE_TYPE_ALL=-1;

    /** the operation to perform on match if the requested resource matches */
    public final static int ACTION_ALLOW = 1,
                            ACTION_DENY = 2;

////////////////////////////////////////////////////////////////////////////////
// data members
////////////////////////////////////////////////////////////////////////////////

    /** the resource type this rule is for, one or more RESOURCE_TYPE_* constants */
    int resourceType = RESOURCE_TYPE_ALL;

    /** the input ip-range (0,0) means ALL !! */
    int ipbegin=0, ipend=0;

    /** the resource (prefix), "" = ALL */
    String resource="";

    /** the action to perform an a match */
    int action = ACTION_DENY;

////////////////////////////////////////////////////////////////////////////////
// construction / initialisation
////////////////////////////////////////////////////////////////////////////////

    /**
     * this method constructs a default rule. This means a rule that blocks
     * ALL accesses
     */
    public AccessFilterRule()
    {
    }

    /**
     * another constructor, for really specifying a rule
     * @param resourceType what resource is this rule for? Use the RESOURCE_TYP_* constants
     * @param ipbegin the first ipnumber (inclusive) in the range for ip-specific rules. (0 means all !!)
     * @param ipend the last ipnumber (inclusive) in the range for ip-specific rules. (0 means all !!)
     * @param resource the prefix of the location that should be matches. Use "" for ALL
     * @param action the action to do if the request matches: ACTION_ALLOW and ACTION_DENY
     */
    public AccessFilterRule( int resourceType, int ipbegin, int ipend, String resource, int action )
    {
        this.resourceType   = resourceType;
        this.ipbegin        = ipbegin;
        this.ipend          = ipend;
        this.resource       = resource;
        this.action         = action;
    }


////////////////////////////////////////////////////////////////////////////////
// request matching
////////////////////////////////////////////////////////////////////////////////

    /**
     * this method checks if the given request matchies this rule
     * @param resourcetype the type of the resource to check. This parameter is required
     *      because we cannot extract the resource type from the request.
     * @param request the request object that occured
     * @return true if the rule matches.
     */
    public boolean matchesRule( int resourceType, HttpRequest request )
    {
        // not the correct type ?? then return
        if( (this.resourceType&resourceType) == 0 ) return false;

        // check the ip-range
        if( (ipbegin==0&&ipend==0) || (ipbegin<=request.getIP4() && request.getIP4()<=ipend) )
        {
            // match the resource
            if( !resource.equals("") )
            {
                if( request.getLocation().startsWith(resource) ) return true;
            }
            return true;
        }
        return false;
    }

    /**
     * this method returns the action associated with this rule
     * @return the ACTION_ALLOW or ACTION_DENY constants
     */
    public int getAction()
    {
        return action;
    }

}
