package com.blommersit.httpd.routes;

import junit.framework.*;
import com.blommersit.httpd.*;

/**
 * Basic unit test for the routes class. We should build in Unit tests for the
 * complete webserver.
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.03
 */
public class TestRoutes extends TestCase
{
	/** the routes data object */
	Routes routes;

	/** test the routes */
	public TestRoutes(String name)
	{
		super(name);
	}

	/** set up test */
	protected void setUp() throws Exception
	{
		super.setUp();
		routes = new Routes();
		routes.add( new Route( "/", "test.Root" ) );
		routes.add( new Route( "/fixed/path/text", "test.FixedPathTest" ) );

		routes.add( new Route( "/static/$controller/$action", "test.ParameterTest" ) );

		Route route = new Route( "/$year/fixed", "test.RequirementsTest" );
		route.setRequirement( "$year", "\\d{4}" );	// 4 decimals
		routes.add( route );
		routes.add( new Route( "/32/fixed", "test.RequirementTestFixed" ) );

		route = new Route( "/default/$a/$b/$c", "test.DefaultTest" );
		route.setDefault( "$a", "aval" );
		route.setDefault( "$b", "bval" );
		route.setDefault( "$c", "cval" );
		routes.add( route );

		routes.add( new Route("/catchall/*", "test.CatchAll" ) );

		routes.add( new Route("/last_slash_test/", "test.LastSlashTest" ) );
	}

	/** and clean up initialisation */
	protected void tearDown() throws Exception
	{
		super.tearDown();
		routes = null;
	}

	/** test a given route */
	protected String getRouteResult( String location )
	{
		HttpRequest request = new HttpRequest( null );	// eeek !!
		request.setLocation( location );
		return routes.findRoute( request );
	}

	protected String getRouteResult( String location, HttpRequest request )
	{
		request.setLocation( location );
		return routes.findRoute( request );
	}


	public void testFindRoute_root()
	{
		// root test
		assertEquals("Root 1", "test.Root", getRouteResult( "/" ) );
		assertEquals("Root 2", "test.Root", getRouteResult( "" ) );
		assertEquals("Root 3", null, getRouteResult( "invalid/url" ) );
	}

	public void testFindRoute_fixed()
	{
		assertEquals("Fixed Path 1", "test.FixedPathTest", getRouteResult( "/fixed/path/text" ) );
		assertEquals("Fixed Path 2", "test.FixedPathTest", getRouteResult( "fixed/path/text" ) );
		assertEquals("Fixed Path 3", "test.FixedPathTest", getRouteResult( "/fixed/path/text/" ) );
		assertEquals("Fixed Path 4", "test.FixedPathTest", getRouteResult( "fixed/path/text/" ) );
		assertEquals("Fixed Path 5", null, getRouteResult( "fixed/path/text/test" ) );
	}

	public void testFindRoute_parameter()
	{
		HttpRequest request = new HttpRequest( null );
		assertEquals("Parameter 1", "test.ParameterTest", getRouteResult( "/static/blog/view", request ) );
		assertEquals(" - GET.controller", "blog", request.getGet( "controller","") );
		assertEquals(" - GET.action", "view", request.getGet( "action","") );
		assertEquals("Parameter 2", null, getRouteResult( "/static/blog/", request ) );
		assertEquals("Parameter 3", null, getRouteResult( "/static/blog//test", request ) );
		request = new HttpRequest( null );
		assertEquals("Parameter 4", "test.ParameterTest", getRouteResult( "/static///", request ) );
		assertEquals(" - GET.controller", "", request.getGet( "controller","") );
		assertEquals(" - GET.action", "", request.getGet( "action","") );
	}

	public void testFindRoute_requirements()
	{
		HttpRequest request = new HttpRequest( null );
		assertEquals("Requirement 1", "test.RequirementsTest", getRouteResult( "/2004/fixed", request ) );
		assertEquals(" - GET.year", "2004", request.getGet( "year","") );

		request = new HttpRequest( null );
		assertEquals("Requirement 2", "test.RequirementsTest", getRouteResult( "/2006/fixed/", request ) );
		assertEquals(" - GET.year", "2006", request.getGet( "year","") );

		request = new HttpRequest( null );
		assertEquals("Requirement 3", "test.RequirementTestFixed", getRouteResult( "/32/fixed", request ) );
		assertEquals(" - GET.year", null, request.getGet( "year", null) );
	}

	public void testFindRoute_defaults()
	{
		HttpRequest request = new HttpRequest( null );
		assertEquals("Default 1", "test.DefaultTest", getRouteResult( "/default/x/y/z", request ) );
		assertEquals(" - GET.a", "x", request.getGet( "a","") );
		assertEquals(" - GET.b", "y", request.getGet( "b","") );
		assertEquals(" - GET.c", "z", request.getGet( "c","") );

		request = new HttpRequest( null );
		assertEquals("Default 2", "test.DefaultTest", getRouteResult( "/default/x/y", request ) );
		assertEquals(" - GET.a", "x", request.getGet( "a","") );
		assertEquals(" - GET.b", "y", request.getGet( "b","") );
		assertEquals(" - GET.c", "cval", request.getGet( "c","") );

		request = new HttpRequest( null );
		assertEquals("Default 3", null, getRouteResult( "/default/x/y/z/test", request ) );

	}

	public void testFindRoute_catchAll()
	{
		assertEquals("Catch All 1", "test.CatchAll", getRouteResult( "/catchall" ) );
		assertEquals("Catch All 2", "test.CatchAll", getRouteResult( "/catchall/a" ) );
		assertEquals("Catch All 3", "test.CatchAll", getRouteResult( "/catchall/a/b" ) );
	}

	public void testFindRoute_lastSlash()
	{
		assertEquals("Last Slash 1", "test.LastSlashTest", getRouteResult( "/last_slash_test" ) );
		assertEquals("Last Slash 2", "test.LastSlashTest", getRouteResult( "/last_slash_test/" ) );
		assertEquals("Last Slash 3", "test.LastSlashTest", getRouteResult( "last_slash_test" ) );
		assertEquals("Last Slash 4", "test.LastSlashTest", getRouteResult( "last_slash_test/" ) );
	}


}
